/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * This file is part of SableCC.                             *
 * See the file "LICENSE" for copyright information and the  *
 * terms and conditions for copying, distribution and        *
 * modification of SableCC.                                  *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

package org.sablecc.sablecc;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

final class LR0Collection
{
  private final List<LR0ItemSet> sets = new ArrayList<>(0);
  private final Map<LR0ItemSet, Integer> setIndices = new TreeMap<>();
  private final List<Map<Symbol, Integer>> GOTO = new ArrayList<>(0);
  final List<String> names = new ArrayList<>(0);

  LR0Collection(LR0ItemSet set
                 )
  {
    System.out.println(" - Computing LR(0) items.");

    add
      (set
          , -1, null);

    for(int i = 0; i < sets.size(); i++)
    {
      System.out.print(".");
      Symbol[] symbols = Symbol.symbols();

      for(int j = 0; j < symbols.length; j++)
      {
        addGoto(i, symbols[j], Grammar.GOTO(set
                                            (i), symbols[j]));
      }
    }
    System.out.println();
  }

  private int add
    (LR0ItemSet set
          , int from, Symbol symbol)
    {
      Integer result = set
                         (set
                         );

      if(result == null)
      {
        result = sets.size();

        setIndices.put(set
                       , result);
        sets.add(set);
        GOTO.add(new TreeMap<Symbol, Integer>());
        if(from == -1)
        {
          names.add(" ");
        }
        else
        {
          names.add(names.get(from) + "" + symbol + " ");
        }
      }

      return result.intValue();
    }

  private static LR0ItemSet empty = new LR0ItemSet();

  public static void reinit()
  {
    empty = new LR0ItemSet();
  }

  private void addGoto(int from, Symbol symbol, LR0ItemSet to)
  {
    if(!to.equals(empty))
    {
      GOTO.get(from).put(symbol, add(to, from, symbol));
    }
  }

  private Integer set
    (LR0ItemSet set
      )
    {
      return setIndices.get(set);
    }

  private LR0ItemSet set
    (int index)
  {
    return sets.get(index);
  }

  LR0ItemSet[] sets()
  {
    return sets.toArray(new LR0ItemSet[0]);
  }

  Integer GOTO(int set
                 , Symbol symbol)
  {
    return GOTO.get(set).get(symbol);
  }

  @Override
  public String toString()
  {
    StringBuilder result = new StringBuilder();

    result.append("{[LR0ItemCollection]");
    result.append(System.getProperty("line.separator"));
    LR0ItemSet[] sets = sets();
    Symbol[] symbols = Symbol.symbols();

    for(int i = 0; i < sets.length; i++)
    {
      result.append(i);
      result.append(":");
      result.append(Grammar.CLOSURE(sets[i]));
      result.append(System.getProperty("line.separator"));

      for(int j = 0; j < symbols.length; j++)
      {
        if(GOTO(i, symbols[j]) != null)
        {
          result.append("[");
          result.append(symbols[j]);
          result.append(":");
          result.append(GOTO(i, symbols[j]));
          result.append("]");
        }
      }

      result.append(System.getProperty("line.separator"));
    }

    result.append("}");
    return result.toString();
  }
}
