/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * This file is part of SableCC.                             *
 * See the file "LICENSE" for copyright information and the  *
 * terms and conditions for copying, distribution and        *
 * modification of SableCC.                                  *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

package org.sablecc.sablecc;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

final class Production
{
  final int leftside;
  final int index;
  final String name;

  private final List<Symbol> rightside = new ArrayList<>();
  private static final List<Production> productions = new ArrayList<>();
  private static Map<Integer, Production[]> alternatives_ = new TreeMap<>();
  private static boolean modified_ = true;
  private static Production[] productions_;

  public static void reinit()
  {
    productions.clear();
    alternatives_ = new TreeMap<>();
    productions_ = null;
    modified_ = true;
    productions_ = null;
  }

  private static void computeArray_()
  {
    productions_ = productions.toArray(new Production[0]);
    modified_ = false;
  }

  private boolean modified = true;
  private Symbol[] rightside_;

  private void computeArray()
  {
    rightside_ = rightside.toArray(new Symbol[0]);
    modified = false;
  }

  Production(int leftside, String name)
  {
    productions.add(this);

    this.name = name;
    this.leftside = leftside;
    this.index = productions.indexOf(this);
    modified_ = true;
  }

  Symbol[] rightside()
  {
    if(modified)
    {
      computeArray();
    }

    return rightside_;
  }

  void addSymbol(Symbol s)
  {
    rightside.add(s);
    modified = true;
    modified_ = true;
  }

  Symbol rightside(int index)
  {
    return rightside.get(index);
  }

  static Production production(int index)
  {
    return productions.get(index);
  }

  static Production[] alternatives(int nonterminal)
  {
    if(modified_)
    {
      alternatives_ = new TreeMap<>();
    }

    Production[] result = alternatives_.get(nonterminal);

    if(result == null)
    {
      List<Production> alternatives = new ArrayList<>(0);

      for(Production production : productions)
      {
        if(production.leftside == nonterminal)
        {
          alternatives.add(production);
        }
      }

      result = alternatives.toArray(new Production[0]);

      alternatives_.put(nonterminal, result);
    }

    return result;
  }

  static Production[] productions()
  {
    if(modified_)
    {
      computeArray_();
    }

    return productions_;
  }

  @Override
  public String toString()
  {
    StringBuilder result = new StringBuilder();

    result.append(Symbol.symbol(leftside, false));

    result.append(" = ");

    Symbol[] symbols = rightside();
    boolean space = false;
    for(int i = 0; i < symbols.length; i++)
    {
      if(space)
      {
        result.append(" ");
      }
      else
      {
        space = true;
      }

      result.append(symbols[i]);
    }

    return result.toString();
  }
}
